==> workbench_approver-1.0.0.tar.gz: workbench_approver/tests/src/Functional/AccessTest.php <==
<?php

namespace Drupal\Tests\workbench_approver\Functional;

use Drupal\workbench_approver\Entity\ApproverInterface;

/**
 * Tests for access to approval pages and tabs.
 *
 * @group workbench_approver
 */
class AccessTest extends WorkbenchApproverTestBase {

  /**
   * Test access to the 'Approve' tab.
   *
   * @covers \Drupal\workbench_approver\Access\ViewApprovalsCheck
   */
  public function testApproveTabAccess() {
    $values = [
      'type' => 'approved',
      'title' => 'Approved node',
      'moderation_state' => 'published',
    ];
    $approved_node = $this->drupalCreateNode($values);

    $values = [
      'type' => 'non_approved',
      'title' => 'Non-approved node',
    ];
    $non_approved_node = $this->drupalCreateNode($values);

    // Testing paths.
    $base_approved_path = '/node/' . $approved_node->id();
    $tab_approved_path = $base_approved_path . '/approver';
    $base_non_approved_path = '/node/' . $non_approved_node->id();
    $tab_non_approved_path = $base_non_approved_path . '/approver';

    // No tab appears unless the node is moderated.
    $users = [
      NULL,
      $this->editorUser,
      $this->adminUser,
      $this->observerUser,
    ];

    foreach ($users as $user) {
      if ($user !== NULL) {
        $this->drupalLogin($user);
      }
      $this->drupalGet($base_approved_path);
      $this->assertSession()->statusCodeEquals(200);
      $this->drupalGet($tab_approved_path);
      $status_code = ($user !== NULL) ? 200 : 403;
      $this->assertSession()->statusCodeEquals($status_code);

      $this->drupalGet($base_non_approved_path);
      $this->assertSession()->statusCodeEquals(200);
      $this->drupalGet($tab_non_approved_path);
      $this->assertSession()->statusCodeEquals(403);
    }

    // Create revisions of the nodes.
    $this->createRevision($approved_node);
    $this->createRevision($non_approved_node);

    // With a revision, the tab appears as appropriate.
    foreach ($users as $user) {
      $user !== NULL ? $this->drupalLogin($user) : $this->drupalLogout();

      $this->drupalGet($base_approved_path);
      $this->assertSession()->statusCodeEquals(200);
      $this->drupalGet($tab_approved_path);
      $status_code = ($user !== NULL) ? 200 : 403;
      $this->assertSession()->statusCodeEquals($status_code);

      $this->drupalGet($base_non_approved_path);
      $this->assertSession()->statusCodeEquals(200);
      $this->drupalGet($tab_non_approved_path);
      $this->assertSession()->statusCodeEquals(403);
    }
  }

  /**
   * Test access to the 'Latest version' tab.
   *
   * @covers \Drupal\workbench_approver\Access\ApproveLatestRevisionCheck
   */
  public function testLatestVersionTabAccess() {
    $values = [
      'type' => 'approved',
      'title' => 'Approved node',
      'moderation_state' => 'published',
    ];
    $approved_node = $this->drupalCreateNode($values);

    $values = [
      'type' => 'non_approved',
      'title' => 'Non-approved node',
    ];
    $non_approved_node = $this->drupalCreateNode($values);

    // Testing paths.
    $base_approved_path = '/node/' . $approved_node->id();
    $tab_approved_path = $base_approved_path . '/latest';
    $base_non_approved_path = '/node/' . $non_approved_node->id();
    $tab_non_approved_path = $base_non_approved_path . '/latest';

    // No tab appears unless the node has a pending revision.
    $users = [
      NULL,
      $this->editorUser,
      $this->adminUser,
      $this->observerUser,
    ];

    foreach ($users as $user) {
      if ($user !== NULL) {
        $this->drupalLogin($user);
      }
      $this->drupalGet($base_approved_path);
      $this->assertSession()->statusCodeEquals(200);
      $this->drupalGet($tab_approved_path);
      $this->assertSession()->statusCodeEquals(403);

      $this->drupalGet($base_non_approved_path);
      $this->assertSession()->statusCodeEquals(200);
      $this->drupalGet($tab_non_approved_path);
      $this->assertSession()->statusCodeEquals(403);
    }

    // Create revisions of the nodes.
    $approved_node = $this->createRevision($approved_node);

    // With a revision, the tab appears as appropriate.
    foreach ($users as $user) {
      $user !== NULL ? $this->drupalLogin($user) : $this->drupalLogout();

      $this->drupalGet($base_approved_path);
      $this->assertSession()->statusCodeEquals(200);
      $this->drupalGet($tab_approved_path);
      // Non-admins cannot view revisions unless assigned.
      if ($user !== NULL && $user->hasRole('admin')) {
        $this->assertSession()->statusCodeEquals(200);
      }
      else {
        $this->assertSession()->statusCodeEquals(403);
      }

      $this->drupalGet($base_non_approved_path);
      $this->assertSession()->statusCodeEquals(200);
      $this->drupalGet($tab_non_approved_path);
      $this->assertSession()->statusCodeEquals(403);
    }

    // Given that a user is assigned, they can access this tab.
    $approver = $this->addApprovalRequest($this->editorUser, $approved_node);

    $this->drupalLogin($this->editorUser);
    $this->drupalGet($tab_approved_path);
    $this->assertSession()->statusCodeEquals(200);

    // Given that a user is unassigned, they can't access this tab.
    // This is an explicit access cache test.
    $approver->delete();
    $this->drupalGet($tab_approved_path);
    $this->assertSession()->statusCodeEquals(403);

    // Given that a user is reassigned, they can access this tab.
    // This is an explicit access cache test.
    $approver = $this->addApprovalRequest($this->editorUser, $approved_node);
    $this->drupalGet($tab_approved_path);
    $this->assertSession()->statusCodeEquals(200);

    // Given that the user is set to APPROVAL_RESCINDED, no access.
    $approver->set('status', ApproverInterface::APPROVAL_RESCINDED);
    $approver->save();
    $this->drupalGet($tab_approved_path);
    $this->assertSession()->statusCodeEquals(403);

    // Given that the user's access is restored, grant access.
    $approver->set('status', ApproverInterface::APPROVAL_PENDING);
    $approver->save();
    $this->drupalGet($tab_approved_path);
    $this->assertSession()->statusCodeEquals(200);
  }

}

==> workbench_approver-1.0.1.tar.gz: workbench_approver/tests/src/Functional/AccessTest.php <==
<?php

namespace Drupal\Tests\workbench_approver\Functional;

use Drupal\workbench_approver\Entity\ApproverInterface;

/**
 * Tests for access to approval pages and tabs.
 *
 * @group workbench_approver
 */
class AccessTest extends WorkbenchApproverTestBase {

  /**
   * Test access to the 'Approve' tab.
   *
   * @covers \Drupal\workbench_approver\Access\ViewApprovalsCheck
   */
  public function testApproveTabAccess() {
    $values = [
      'type' => 'approved',
      'title' => 'Approved node',
      'moderation_state' => 'published',
    ];
    $approved_node = $this->drupalCreateNode($values);

    $values = [
      'type' => 'non_approved',
      'title' => 'Non-approved node',
    ];
    $non_approved_node = $this->drupalCreateNode($values);

    // Testing paths.
    $base_approved_path = '/node/' . $approved_node->id();
    $tab_approved_path = $base_approved_path . '/approver';
    $base_non_approved_path = '/node/' . $non_approved_node->id();
    $tab_non_approved_path = $base_non_approved_path . '/approver';

    // No tab appears unless the node is moderated.
    $users = [
      NULL,
      $this->editorUser,
      $this->adminUser,
      $this->observerUser,
    ];

    foreach ($users as $user) {
      if ($user !== NULL) {
        $this->drupalLogin($user);
      }
      $this->drupalGet($base_approved_path);
      $this->assertSession()->statusCodeEquals(200);
      $this->drupalGet($tab_approved_path);
      $status_code = ($user !== NULL) ? 200 : 403;
      $this->assertSession()->statusCodeEquals($status_code);

      $this->drupalGet($base_non_approved_path);
      $this->assertSession()->statusCodeEquals(200);
      $this->drupalGet($tab_non_approved_path);
      $this->assertSession()->statusCodeEquals(403);
    }

    // Create revisions of the nodes.
    $this->createRevision($approved_node);
    $this->createRevision($non_approved_node);

    // With a revision, the tab appears as appropriate.
    foreach ($users as $user) {
      $user !== NULL ? $this->drupalLogin($user) : $this->drupalLogout();

      $this->drupalGet($base_approved_path);
      $this->assertSession()->statusCodeEquals(200);
      $this->drupalGet($tab_approved_path);
      $status_code = ($user !== NULL) ? 200 : 403;
      $this->assertSession()->statusCodeEquals($status_code);

      $this->drupalGet($base_non_approved_path);
      $this->assertSession()->statusCodeEquals(200);
      $this->drupalGet($tab_non_approved_path);
      $this->assertSession()->statusCodeEquals(403);
    }
  }

  /**
   * Test access to the 'Latest version' tab.
   *
   * @covers \Drupal\workbench_approver\Access\ApproveLatestRevisionCheck
   */
  public function testLatestVersionTabAccess() {
    $values = [
      'type' => 'approved',
      'title' => 'Approved node',
      'moderation_state' => 'published',
    ];
    $approved_node = $this->drupalCreateNode($values);

    $values = [
      'type' => 'non_approved',
      'title' => 'Non-approved node',
    ];
    $non_approved_node = $this->drupalCreateNode($values);

    // Testing paths.
    $base_approved_path = '/node/' . $approved_node->id();
    $tab_approved_path = $base_approved_path . '/latest';
    $base_non_approved_path = '/node/' . $non_approved_node->id();
    $tab_non_approved_path = $base_non_approved_path . '/latest';

    // No approval tab appears unless the node has a pending revision.
    $users = [
      NULL,
      $this->editorUser,
      $this->adminUser,
      $this->observerUser,
    ];

    foreach ($users as $user) {
      if ($user !== NULL) {
        $this->drupalLogin($user);
      }
      $this->drupalGet($base_approved_path);
      $this->assertSession()->statusCodeEquals(200);
      $this->drupalGet($tab_approved_path);
      // There must be a revision to have a latest tab.
      $this->assertSession()->statusCodeEquals(403);

      $this->drupalGet($base_non_approved_path);
      $this->assertSession()->statusCodeEquals(200);
      $this->drupalGet($tab_non_approved_path);
      $this->assertSession()->statusCodeEquals(403);
    }

    // Create revisions of the nodes.
    $approved_node = $this->createRevision($approved_node);

    // With a revision, the tab appears as appropriate.
    foreach ($users as $user) {
      $user !== NULL ? $this->drupalLogin($user) : $this->drupalLogout();

      $this->drupalGet($base_approved_path);
      $this->assertSession()->statusCodeEquals(200);
      $this->drupalGet($tab_approved_path);
      // Non-admins cannot view revisions unless assigned.
      if ($user !== NULL && $user->hasPermission('view any unpublished content')) {
        $this->assertSession()->statusCodeEquals(200);
      }
      else {
        $this->assertSession()->statusCodeEquals(403);
      }

      $this->drupalGet($base_non_approved_path);
      $this->assertSession()->statusCodeEquals(200);
      $this->drupalGet($tab_non_approved_path);
      $this->assertSession()->statusCodeEquals(403);
    }

    // Given that a user is assigned, they can access this tab.
    $approver = $this->addApprovalRequest($this->editorUser, $approved_node);

    $this->drupalLogin($this->editorUser);
    $this->drupalGet($tab_approved_path);
    $this->assertSession()->statusCodeEquals(200);

    // Given that a user is unassigned, they can't access this tab.
    // This is an explicit access cache test.
    $approver->delete();
    $this->drupalGet($tab_approved_path);
    $this->assertSession()->statusCodeEquals(403);

    // Given that a user is reassigned, they can access this tab.
    // This is an explicit access cache test.
    $approver = $this->addApprovalRequest($this->editorUser, $approved_node);
    $this->drupalGet($tab_approved_path);
    $this->assertSession()->statusCodeEquals(200);

    // Given that the user is set to APPROVAL_RESCINDED, no access.
    $approver->set('status', ApproverInterface::APPROVAL_RESCINDED);
    $approver->save();
    $this->drupalGet($tab_approved_path);
    $this->assertSession()->statusCodeEquals(403);

    // Given that the user's access is restored, grant access.
    $approver->set('status', ApproverInterface::APPROVAL_PENDING);
    $approver->save();
    $this->drupalGet($tab_approved_path);
    $this->assertSession()->statusCodeEquals(200);
  }

}
